/*
 
 Copyright (C) 2006 NTT DATA Corporation
 
 This program is free software; you can redistribute it and/or
 Modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation, version 2.
 
 This program is distributed in the hope that it will be
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 */

package com.clustercontrol.jobmanagement.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.clustercontrol.commons.util.ConnectionManager;
import com.clustercontrol.jobmanagement.ejb.entity.JobSessionBean;
import com.clustercontrol.jobmanagement.ejb.entity.JobSessionPK;

/**
 * ジョブセッションのDAOインターフェースを実装するクラス<BR>
 * 
 * @version 2.4.0
 * @since 2.1.0
 * 
 * @see com.clustercontrol.jobmanagement.ejb.entity.JobSessionBean
 * @see com.clustercontrol.jobmanagement.dao.JobSessionDAO
 */
public class JobSessionDAOImpl implements JobSessionDAO {
	
	/** ログ出力のインスタンス。 */
	protected static Log m_log = LogFactory.getLog(JobSessionDAOImpl.class);
	
	public void init() {
	}
	
	/** 
	 * SELECT文を発行します。
	 * 引数で指定されたプライマリキーで検索し、取得したデータベースの内容をEntity Beanに反映します。
	 * 
	 * @see com.clustercontrol.jobmanagement.dao.JobSessionDAO#load(com.clustercontrol.jobmanagement.ejb.entity.JobSessionPK, com.clustercontrol.jobmanagement.ejb.entity.JobSessionBean)
	 */
	public void load(JobSessionPK pk, JobSessionBean ejb) throws EJBException {
		m_log.debug("load() start : " + pk.toString());
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet res = null;
		
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();
			
			//SQL文の定義
			String sql = "SELECT * FROM cc_job_session WHERE session_id = ?";
			//SQL文のセット
			stmt = conn.prepareStatement(sql);
			
			stmt.setString(1, pk.getSession_id());
			
			res = stmt.executeQuery();
			
			if (res.next()) {
				//取得した値をBeanにセット
				ejb.setJob_id(res.getString("job_id"));
				ejb.setJobunit_id(res.getString("jobunit_id"));
				ejb.setOperation_flg(res.getInt("operation_flg"));
				ejb.setTrigger_type(res.getInt("trigger_type"));
				ejb.setTrigger_info(res.getString("trigger_info"));
				if(res.getTimestamp("schedule_date") != null)
					ejb.setSchedule_date(new Date(res.getTimestamp("schedule_date").getTime()));
				else
					ejb.setSchedule_date(null);
				ejb.setSession_id(res.getString("session_id"));
			} else {
				String msg = "JobSession data is not found.";
				m_log.error("load() error : " + pk.toString() + " SQLException " + msg);
				throw new EJBException(msg);
			}
		} catch (SQLException e) {
			m_log.error("load() error : " + pk.toString() + " SQLException");
			throw new EJBException(e.getMessage());
		} finally {
			//コネクション、結果セット、プリペアドステートメントのクローズ
			try {
				if (stmt != null) {
					stmt.close();
				}
				if (res != null) {
					res.close();
				}
				if (conn != null) {
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("load() error : " + pk.toString() + " SQLException " + e1.getMessage());
				throw new EJBException(e1.getMessage());
			}
		}
		m_log.debug("load() end : " + pk.toString());
	}
	
	/**
	 * UPDATE文を発行します。
	 * 引数で指定されたEntity Beanの内容でデータベースを更新します。
	 * 
	 * @see com.clustercontrol.jobmanagement.dao.JobSessionDAO#store(com.clustercontrol.jobmanagement.ejb.entity.JobSessionBean)
	 */
	public void store(JobSessionBean ejb) throws EJBException {
		m_log.debug("store() start : sessionId=" + ejb.getSession_id() + ", jobunitId=" + ejb.getJobunit_id());
		
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();
			//SQL文の定義
			StringBuilder sql = new StringBuilder();
			sql.append("UPDATE cc_job_session SET ");
			sql.append("job_id = ?, ");
			sql.append("schedule_date = ?, ");
			sql.append("operation_flg = ?, ");
			sql.append("trigger_type = ?, ");
			sql.append("trigger_info = ? ");
			sql.append("WHERE session_id = ? ");
			sql.append("AND jobunit_id = ?");
			
			//SQL文のセット
			stmt = conn.prepareStatement(sql.toString());
			
			stmt.setString(1, ejb.getJob_id());
			if(ejb.getSchedule_date() instanceof java.util.Date)
				stmt.setTimestamp(2, new Timestamp(ejb.getSchedule_date().getTime()));
			else
				stmt.setTimestamp(2, null);
			stmt.setInt(3, ejb.getOperation_flg());
			stmt.setInt(4, ejb.getTrigger_type());
			stmt.setString(5, ejb.getTrigger_info());
			stmt.setString(6, ejb.getSession_id());
			stmt.setString(7, ejb.getJobunit_id());
			
			int row = stmt.executeUpdate();
			
			if (row != 1) {
				String msg = "result row is not 1";
				m_log.error("store() error : sessionId=" + ejb.getSession_id() + ", jobunitId=" + ejb.getJobunit_id() + " SQLException" + msg);
				throw new EJBException(msg);
			}
		} catch (SQLException e) {
			m_log.error("store() error : sessionId=" + ejb.getSession_id() + ", jobunitId=" + ejb.getJobunit_id() + " SQLException");
			throw new EJBException(e.getMessage());
		} finally {
			// コネクション、結果セット、プリペアドステートメントのクローズ
			try {
				if (stmt != null) {
					stmt.close();
				}
				if (conn != null) {
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("store() error : sessionId=" + ejb.getSession_id() + ", jobunitId=" + ejb.getJobunit_id() + " SQLException");
				throw new EJBException(e1.getMessage());
			}
		}
		m_log.debug("store() end : sessionId=" + ejb.getSession_id() + ", jobunitId=" + ejb.getJobunit_id());
	}
	
	/**
	 * DELETE文を発行します。
	 * 引数で指定されたプライマリキーでデータベースから削除します。
	 * 
	 * @see com.clustercontrol.jobmanagement.dao.JobSessionDAO#remove(com.clustercontrol.jobmanagement.ejb.entity.JobSessionPK)
	 */
	public void remove(JobSessionPK pk) throws RemoveException, EJBException {
		m_log.debug("remove() start : " + pk.toString());
		
		Connection conn = null;
		PreparedStatement stmt = null;
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();
			
			//SQL文の定義
			String sql = "DELETE FROM cc_job_session WHERE session_id = ?";
			//SQL文のセット
			stmt = conn.prepareStatement(sql);
			
			stmt.setString(1, pk.getSession_id());
			
			int row = stmt.executeUpdate();
			
			if (row != 1 ) {
				String msg = "result row is not 1";
				m_log.error("remove() error : " + pk.toString() + " SQLException" + msg);
				throw new EJBException(msg);	
			}
		} catch (SQLException e) {
			m_log.error("remove() error : " + pk.toString() + " SQLException");
			throw new EJBException(e.getMessage());
			
		} finally {
			try {
				if(stmt != null){
					stmt.close();
				}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("remove() error : " + pk.toString() + " SQLException");
				throw new EJBException(e1.getMessage());
			}
		}
		m_log.debug("remove() end : " + pk.toString());
	}
	
	/**
	 * INSERT文を発行します。
	 * 引数で指定されたEntity Beanの内容をデータベースに挿入し、プライマリキーを返します。
	 * 
	 * @see com.clustercontrol.jobmanagement.dao.JobSessionDAO#create(com.clustercontrol.jobmanagement.ejb.entity.JobSessionBean)
	 */
	public JobSessionPK create(JobSessionBean ejb) throws CreateException, EJBException {
		m_log.debug("create() start : sessionId=" + ejb.getSession_id() + ", jobunitId=" + ejb.getJobunit_id());
		
		Connection conn = null;
		PreparedStatement stmt = null;
		JobSessionPK pk;
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();
			//SQL文の定義
			StringBuilder sql = new StringBuilder();
			sql.append("INSERT INTO cc_job_session (");
			sql.append("session_id, ");
			sql.append("job_id, ");
			sql.append("schedule_date, ");
			sql.append("operation_flg, ");
			sql.append("trigger_type, ");
			sql.append("trigger_info, ");
			sql.append("jobunit_id) ");
			sql.append("VALUES (?,?,?,?,?,?,?)");
			
			//SQL文のセット
			stmt = conn.prepareStatement(sql.toString());
			
			stmt.setString(1, ejb.getSession_id());
			stmt.setString(2, ejb.getJob_id());
			if(ejb.getSchedule_date() instanceof java.util.Date)
				stmt.setTimestamp(3, new Timestamp(ejb.getSchedule_date().getTime()));
			else
				stmt.setTimestamp(3, null);
			stmt.setInt(4, ejb.getOperation_flg());
			stmt.setInt(5, ejb.getTrigger_type());
			stmt.setString(6, ejb.getTrigger_info());
			stmt.setString(7, ejb.getJobunit_id());
			
			int row =	stmt.executeUpdate();
			pk = new JobSessionPK(
					ejb.getSession_id());
			
			if (row != 1) {
				String msg = "result row is not 1";
				m_log.error("create() error : sessionId=" + ejb.getSession_id() + ", jobunitId=" + ejb.getJobunit_id() + " SQLException");
				throw new EJBException(msg);
			}
		} catch (SQLException e) {
			if(e.getSQLState().equals("23505")){
				m_log.error("create() error : sessionId=" + ejb.getSession_id() + ", jobunitId=" + ejb.getJobunit_id() + " SQLException" + "DuplicateKeyException ");
				throw new DuplicateKeyException(e.getMessage());
			}
			throw new CreateException(e.getMessage());
		}finally{
			try {
				if(stmt != null){
					stmt.close();
				}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("create() error : sessionId=" + ejb.getSession_id() + ", jobunitId=" + ejb.getJobunit_id() + " SQLException");
				throw new EJBException(e1.getMessage());
			}
		}
		m_log.debug("create() end : sessionId=" + ejb.getSession_id() + ", jobunitId=" + ejb.getJobunit_id());
		return pk;
	}
	
	/**
	 * 全件取得するSELECT文を発行します。
	 * 取得したデータのプライマリキーをコレクションに格納し返します。
	 * 
	 * @see com.clustercontrol.jobmanagement.ejb.entity.JobSessionBean#ejbFindAll()
	 * @see com.clustercontrol.jobmanagement.dao.JobSessionDAO#findAll()
	 */
	public Collection findAll() throws FinderException {
		m_log.debug("findAll() start");
		
		ArrayList<JobSessionPK> ret = new ArrayList<JobSessionPK>();
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet res = null;
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();	
			//SQL文の定義
			StringBuffer sql = new StringBuffer();
			sql.append("SELECT session_id, jobunit_id FROM cc_job_session");
			
			//SQL文のセット
			stmt = conn.prepareStatement(sql.toString());
			res = stmt.executeQuery();
			
			while(res.next()) {
				JobSessionPK pk= new JobSessionPK(
						res.getString("session_id")
				);
				ret.add(pk);
			}
			
		} catch (SQLException e) {
			m_log.error("findAll() error :  SQLException");
			throw new EJBException(e.getMessage());
		}  finally{
			try {
				if(stmt != null){
					stmt.close();}
				if(res != null){
					res.close();}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("findAll() error :  SQLException");
				throw new EJBException(e1.getMessage());
			}
		}
		m_log.debug("findAll() end");
		return ret;
	}
	
	/**
	 * 1件取得するSELECT文を発行します。
	 * 引数で指定されたプライマリキーで、データベースを検索します。
	 * 
	 * @see com.clustercontrol.jobmanagement.ejb.entity.JobSessionBean#ejbFindByPrimaryKey(com.clustercontrol.jobmanagement.ejb.entity.JobSessionPK)
	 * @see com.clustercontrol.jobmanagement.dao.JobSessionDAO#findByPrimaryKey(com.clustercontrol.jobmanagement.ejb.entity.JobSessionPK)
	 */
	public JobSessionPK findByPrimaryKey(JobSessionPK pk) throws FinderException {
		m_log.debug("findByPrimaryKey() start : " + pk.toString());
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet res = null;
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();
			
			//SQL文の定義
			String sql = "SELECT session_id, jobunit_id FROM cc_job_session WHERE session_id = ?";
			//SQL文のセット
			stmt = conn.prepareStatement(sql);
			
			stmt.setString(1, pk.getSession_id());
			
			res = stmt.executeQuery();
			
			if (res.next()) {
				return pk;
			} else {
				String msg = "id " + pk.toString() + " not found.";
				throw new FinderException(msg);
			}
		} catch (SQLException e) {
			m_log.error("find() error : " + pk.toString() + " SQLException");
			throw new EJBException(e.getMessage());
		}  finally{
			try {
				if(stmt != null){
					stmt.close();}
				if(res != null){
					res.close();}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("find() error : " + pk.toString() + " SQLException");
				throw new EJBException(e1.getMessage());
			}
			m_log.debug("findByPrimaryKey() end : " + pk.toString());
		}
	}
	
	/**
	 * 複数件取得するSELECT文を発行します。
	 * 引数で指定された値で、データベースを検索します。<BR>
	 * 引数：ログインユーザのユーザID、開始日時FROM、開始日時TO、終了日時FROM、終了日時TO、ジョブID、実行状態、最大数
	 * 
	 * @see com.clustercontrol.jobmanagement.ejb.entity.JobSessionBean#ejbFindByPrimaryKey(com.clustercontrol.jobmanagement.ejb.entity.JobSessionPK)
	 * @see com.clustercontrol.jobmanagement.dao.JobSessionDAO#findByPrimaryKey(com.clustercontrol.jobmanagement.ejb.entity.JobSessionPK)
	 */
	public Collection findHistory(String userId, Date startFromDate, Date startToDate, Date endFromDate, Date endToDate, 
			String jobId, Integer status, Integer triggerType, String triggerInfo, Integer limit) throws FinderException {
		
		m_log.debug("findHistory() start : " + startFromDate + ", " + startToDate + ", " + endFromDate + ", " + endToDate + ", " + 
				jobId + ", " + status + ", " + triggerType + ", " + triggerInfo + ", " + limit);
		
		ArrayList<JobSessionPK> ret = new ArrayList<JobSessionPK>();
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet res = null;
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();	
			//SQL文の定義
			StringBuilder sql = new StringBuilder();
			
			String sql_session;  // ログインユーザで参照可能なセッション情報
			String sql_jobunit1; // ログインユーザが参照できないジョブユニットID
			String sql_jobunit2; // ログインユーザが参照可能なジョブニットID
			
			sql_jobunit2 = "SELECT m.jobunit_id FROM cc_job_mst m LEFT OUTER JOIN cc_job_user_relation_mst u ON m.jobunit_id = u.jobunit_id WHERE m.job_type = 0 AND (u.user_id = '" + userId + "' OR u.user_id IS NULL) ";
			sql_jobunit1 = "SELECT DISTINCT jobunit_id FROM cc_job_mst WHERE job_type = 0 AND jobunit_id NOT IN ( "
					+ sql_jobunit2 + " )";
			sql_session = "SELECT a.session_id, a.jobunit_id FROM cc_job_session as a, cc_job_session_job as b WHERE a.session_id = b.session_id AND a.job_id = b.job_id AND a.jobunit_id NOT IN ( "
				+ sql_jobunit1 + " ) ";
			
			sql.append(sql_session);
			
			if(startFromDate instanceof Date) {
				sql.append(" AND b.start_date >= ?");
			}
			if(startToDate instanceof Date) {
				sql.append(" AND b.start_date <= ?");
			}
			if(endFromDate instanceof Date) {
				sql.append(" AND b.end_date >= ?");
			}
			if(endToDate instanceof Date) {
				sql.append(" AND b.end_date <= ?");
			}
			if(jobId != null && jobId.length() > 0) {
				sql.append(" AND b.job_id = ?");
			}
			if(status instanceof Integer && status.intValue() != -1) {
				sql.append(" AND b.status = ?");
			}
			if(triggerType instanceof Integer && triggerType.intValue() != -1) {
				sql.append(" AND a.trigger_type = ?");
			}
			if(triggerInfo != null && triggerInfo.length() > 0) {
				sql.append(" AND a.trigger_info like ?");
			}
			sql.append(" ORDER BY a.session_id DESC");
			if(limit != null){
				sql.append(" LIMIT " + limit.toString());
			}
			
			//SQL文のセット
			stmt = conn.prepareStatement(sql.toString());
			
			int count = 1;
					if(startFromDate instanceof Date) {
				stmt.setTimestamp(count, new Timestamp(startFromDate.getTime()));
				count++;
			}
			if(startToDate instanceof Date) {
				stmt.setTimestamp(count, new Timestamp(startToDate.getTime()));
				count++;
			}
			if(endFromDate instanceof Date) {
				stmt.setTimestamp(count, new Timestamp(endFromDate.getTime()));
				count++;
			}
			if(endToDate instanceof Date) {
				stmt.setTimestamp(count, new Timestamp(endToDate.getTime()));
				count++;
			}
			if(jobId != null && jobId.length() > 0) {
				stmt.setString(count, jobId);
				count++;
			}
			if(status instanceof Integer && status.intValue() != -1) {
				stmt.setInt(count, status);
				count++;
			}
			if(triggerType instanceof Integer && triggerType.intValue() != -1) {
				stmt.setInt(count, triggerType);
				count++;
			}
			if(triggerInfo != null && triggerInfo.length() > 0) {
				stmt.setString(count, "%" + triggerInfo + "%");
				count++;
			}
			
			m_log.debug("findHistory() : sql=" + sql.toString());
			res = stmt.executeQuery();
			
			while(res.next()) {
				JobSessionPK pk= new JobSessionPK(
						res.getString("session_id")
				);
				ret.add(pk);
			}
			
		} catch (SQLException e) {
			m_log.error("findHistory() error : " + startFromDate + ", " + startToDate + ", " + endFromDate + ", " + endToDate + ", " + 
					jobId + ", " + status + ", " + triggerType + ", " + triggerInfo + ", " + limit + " SQLException");
			throw new EJBException(e.getMessage());
		}  finally{
			try {
				if(stmt != null){
					stmt.close();}
				if(res != null){
					res.close();}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("findHistory() error : " + startFromDate + ", " + startToDate + ", " + endFromDate + ", " + endToDate + ", " + 
						jobId + ", " + status + ", " + triggerType + ", " + triggerInfo + ", " + limit + " SQLException");
				throw new EJBException(e1.getMessage());
			}
		}
		m_log.debug("findHistory() end : " + startFromDate + ", " + startToDate + ", " + endFromDate + ", " + endToDate + ", " + 
				jobId + ", " + status + ", " + triggerType + ", " + triggerInfo + ", " + limit);
		return ret;
	}
	
	/**
	 * レコード数を取得するSELECT文を発行します。
	 * 引数で指定された値で、データベースを検索します。<BR>
	 * 
	 * @param startFromDate 開始日時FROM
	 * @param startToDate 開始日時TO
	 * @param endFromDate 終了日時FROM
	 * @param endToDate 終了日時TO
	 * @param jobId ジョブID
	 * @param status 実行状態
	 * @param triggerType 実行契機種別
	 * @param triggerInfo 実行契機情報
	 * @return レコード数
	 * @throws SQLException
	 * 
	 * @see com.clustercontrol.jobmanagement.ejb.entity.JobSessionBean#ejbFindHistory(Date, Date, Date, Date, String, Integer, Integer, String, Integer)
	 */
	public int countHistory(Date startFromDate, Date startToDate, Date endFromDate, Date endToDate, 
			String jobId, Integer status, Integer triggerType, String triggerInfo) throws SQLException {
		m_log.debug("countHistory() start : " + startFromDate + ", " + startToDate + ", " + endFromDate + ", " + endToDate + ", " + 
				jobId + ", " + status + ", " + triggerType + ", " + triggerInfo);
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet res = null;
		int total = 0;
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();	
			//SQL文の定義
			StringBuilder sql = new StringBuilder();
			sql.append("SELECT COUNT(*) FROM cc_job_session as a, cc_job_session_job as b ");
			sql.append("WHERE a.session_id = b.session_id AND a.job_id = b.job_id");
			if(startFromDate instanceof Date) {
				sql.append(" AND b.start_date >= ?");
			}
			if(startToDate instanceof Date) {
				sql.append(" AND b.start_date <= ?");
			}
			if(endFromDate instanceof Date) {
				sql.append(" AND b.end_date >= ?");
			}
			if(endToDate instanceof Date) {
				sql.append(" AND b.end_date <= ?");
			}
			if(jobId != null && jobId.length() > 0) {
				sql.append(" AND b.job_id = ?");
			}
			if(status instanceof Integer && status.intValue() != -1) {
				sql.append(" AND b.status = ?");
			}
			if(triggerType instanceof Integer && triggerType.intValue() != -1) {
				sql.append(" AND a.trigger_type = ?");
			}
			if(triggerInfo != null && triggerInfo.length() > 0) {
				sql.append(" AND a.trigger_info like ?");
			}
			
			//SQL文のセット
			stmt = conn.prepareStatement(sql.toString());
			
			int count = 1;
			if(startFromDate instanceof Date) {
				stmt.setTimestamp(count, new Timestamp(startFromDate.getTime()));
				count++;
			}
			if(startToDate instanceof Date) {
				stmt.setTimestamp(count, new Timestamp(startToDate.getTime()));
				count++;
			}
			if(endFromDate instanceof Date) {
				stmt.setTimestamp(count, new Timestamp(endFromDate.getTime()));
				count++;
			}
			if(endToDate instanceof Date) {
				stmt.setTimestamp(count, new Timestamp(endToDate.getTime()));
				count++;
			}
			if(jobId != null && jobId.length() > 0) {
				stmt.setString(count, jobId);
				count++;
			}
			if(status instanceof Integer && status.intValue() != -1) {
				stmt.setInt(count, status);
				count++;
			}
			if(triggerType instanceof Integer && triggerType.intValue() != -1) {
				stmt.setInt(count, triggerType);
				count++;
			}
			if(triggerInfo != null && triggerInfo.length() > 0) {
				stmt.setString(count, "%" + triggerInfo + "%");
				count++;
			}
			
			res = stmt.executeQuery();
			
			if(res.next()) {
				total = res.getInt(1);
			}
			
		} catch (SQLException e) {
			m_log.error("countHistory() error : " + startFromDate + ", " + startToDate + ", " + endFromDate + ", " + endToDate + ", " + 
					jobId + ", " + status + ", " + triggerType + ", " + triggerInfo + " SQLException");
			throw e;
		}  finally{
			try {
				if(stmt != null){
					stmt.close();}
				if(res != null){
					res.close();}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("countHistory() error : " + startFromDate + ", " + startToDate + ", " + endFromDate + ", " + endToDate + ", " + 
						jobId + ", " + status + ", " + triggerType + ", " + triggerInfo + " SQLException");
				throw e1;
			}
		}
		m_log.debug("countHistory() end : " + startFromDate + ", " + startToDate + ", " + endFromDate + ", " + endToDate + ", " + 
				jobId + ", " + status + ", " + triggerType + ", " + triggerInfo);
		return total;
	}
	
	/**
	 * 未終了のセッション情報を返します。
	 * @return
	 * @throws FinderException
	 */
	public Collection findUnendSessions() throws FinderException {
		m_log.debug("findUnendSessions() start");
		
		ArrayList<JobSessionPK> ret = new ArrayList<JobSessionPK>();
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet res = null;
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();	
			//SQL文の定義
			String sql = "SELECT session_id FROM cc_job_session NATURAL JOIN cc_job_session_job " +
					"WHERE status NOT IN (300,301) ORDER BY start_date ASC";
			
			//SQL文のセット
			stmt = conn.prepareStatement(sql);
			
			m_log.debug("findUnendSessions() : sql = " + sql);
			res = stmt.executeQuery();
			
			while(res.next()) {
				JobSessionPK pk= new JobSessionPK(
						res.getString("session_id")
				);
				ret.add(pk);
			}
			
		} catch (SQLException e) {
			m_log.error("findUnendSessions() error ",e);
			throw new EJBException(e.getMessage());
			
		}  finally{
			try {
				if(stmt != null){
					stmt.close();}
				if(res != null){
					res.close();}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("findUnendSessions() error ",e1);
				throw new EJBException(e1.getMessage());
			}
		}
		m_log.debug("findUnendSessions() end");
		return ret;
	}
}
