/*
 
 Copyright (C) 2006 NTT DATA Corporation
 
 This program is free software; you can redistribute it and/or
 Modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation, version 2.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 */

package com.clustercontrol.performance.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.ejb.EJBException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.clustercontrol.commons.util.ConnectionManager;
import com.clustercontrol.performance.bean.CollectedDataInfo;

/**
 * 収集した性能値をDBに保存、読み出しするDAOクラス
 * 
 * 
 * @version 2.0.0
 * @since 1.0
 */
public class CalculatedDataDAO {
	
//	ログ出力
	protected static Log m_log = LogFactory.getLog(CalculatedDataDAO.class);
	
	/**
	 * コンストラクター
	 */
	public CalculatedDataDAO(){
	}
	
	/**
	 * 収集された性能値を収集ID、SNMPのOID、ファシリティID、期間（開始の時刻、終了の時刻）で検索します。
	 *
	 * 
	 * @param collectorID 収集ID
	 * @param itemCode 収集項目コード
	 * @param displayName リポジトリ表示名
	 * @param facilityID ファシリティID
	 * @param startDate (検索の）開始時刻
	 * @param stopDate （検索の）終了時刻
	 * @return　収集された性能値のコレクション
	 * @throws EJBException
	 */
	public List<CollectedDataInfo> select(
			String collectorID, 
//			String collectMethod,
			String itemCode,
			String displayName,
			String facilityID,
			Date startDate,
			Date stopDate )
	throws EJBException{
		
		m_log.debug("select() start :" + collectorID + " " + itemCode + " " + displayName + " " + facilityID);
		ArrayList<CollectedDataInfo> ret = new ArrayList<CollectedDataInfo>();  // DBから取得したデータを格納するリスト
		
		Connection conn = null;
		PreparedStatement stmt =null;
		ResultSet res = null; 
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();
			
			//		SQL文の作成
			String sql = "SELECT DATE_TIME,VALUE FROM CC_CALCULATED_DATA WHERE " + 
			"COLLECTORID=? AND " +
//			"COLLECT_METHOD=? AND " +
			"ITEM_CODE=? AND " + 
			"DISPLAY_NAME=? AND " + 
			"FACILITYID=? AND " + 
			"DATE_TIME BETWEEN ? AND ?" +
			"ORDER BY DATE_TIME";
			
			// SQL文発行準備
			stmt = conn.prepareStatement(sql);
			
			Timestamp start = new Timestamp(startDate.getTime());
			Timestamp stop  = new Timestamp(stopDate.getTime());
			
			stmt.setString(1, collectorID);
			stmt.setString(2, itemCode);
			stmt.setString(3, displayName);
			stmt.setString(4, facilityID);
			stmt.setTimestamp(5, start);
			stmt.setTimestamp(6, stop);
			
			res = stmt.executeQuery();
			
			while(res.next()) {
				Date d = new Date(res.getTimestamp("DATE_TIME").getTime());
				CollectedDataInfo data;
				if(res.getObject("VALUE") != null){
					data = new CollectedDataInfo(facilityID, itemCode, displayName, d, res.getDouble("VALUE"));
				} else {
					data = new CollectedDataInfo(facilityID, itemCode, displayName, d, Double.NaN);
				}
				
//				CollectedDataInfo data = new CollectedDataInfo(facilityID, itemCode, deviceName, collectMethod);
//				data.setDate(new Date(res.getTimestamp("DATE_TIME").getTime()));
//				
//				if(res.getObject("VALUE") != null){	
//					data.setValue(res.getDouble("VALUE"));
//				} else {
//					data.setValue(Double.NaN);  // DBにnullが入っている場合は性能値が算出不能だった場合
//				}
				ret.add(data);
			}
		} catch (SQLException e) {
			m_log.error("select() error :" + facilityID + " EJBException ",e);
			throw new EJBException(e.getMessage());
		}  finally{
			
			//コネクション、結果セット、プリペアドステートメントのクロース
			try {
				if(res != null){
					res.close();
				}
				if(stmt != null){
					stmt.close();
				}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("select() error :" + facilityID + " EJBException ");
				throw new EJBException(e1.getMessage());
			}
		}	
		m_log.debug("select() end   :" + collectorID + " " + itemCode + " " + displayName + " " + facilityID);
		
		return ret;
	}
	
	/**
	 * 実績収集結果を収集ID、ファシリティID、検索期間（開始日時および終了日時）の組み合わせで検索し、収集性能値DTOクラスの配列として返す。
	 * 
	 * @param collectorID
	 *            収集ID
	 * @param facilityID
	 *            ファシリティID
	 * @param startDate
	 *            検索対象となる開始日時
	 * @param stopDate
	 *            検索対象となる終了日時
	 * @return 該当する収集性能値DTOクラスの配列（順序は収集日時順とは限らない）
	 * @throws EJBException
	 * 
	 * @author NTT DATA takahatat
	 */
	public ArrayList<CollectedDataInfo> select(String collectorID, String facilityID, Date startDate, Date stopDate)
			throws EJBException {

		m_log.debug("select() start :" + collectorID + " " + facilityID);
		ArrayList<CollectedDataInfo> ret = new ArrayList<CollectedDataInfo>();
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet res = null;
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();

			// SQL文の作成
			String sql = "SELECT date_time, item_code, display_name, value FROM cc_calculated_data WHERE collectorid = ? AND facilityid = ? AND date_time >= ? AND date_time < ?";

			// SQL文発行準備
			stmt = conn.prepareStatement(sql);

			Timestamp startTimestamp = new Timestamp(startDate.getTime());
			Timestamp stopTimestamp = new Timestamp(stopDate.getTime());

			stmt.setString(1, collectorID);
			stmt.setString(2, facilityID);
			stmt.setTimestamp(3, startTimestamp);
			stmt.setTimestamp(4, stopTimestamp);

			res = stmt.executeQuery();

			// 配列への性能値の格納
			while (res.next()) {
				if (res.getObject("VALUE") != null) {
					ret.add(new CollectedDataInfo(facilityID, res.getString("ITEM_CODE"), res.getString("DISPLAY_NAME"), res.getTimestamp("DATE_TIME"), res.getDouble("VALUE")));
				}
			}
		} catch (SQLException e) {
			m_log.error("select() error :" + collectorID + " " + facilityID + " EJBException ", e);
			throw new EJBException(e.getMessage());
		} finally {

			// コネクション、結果セット、プリペアドステートメントのクロース
			try {
				if (res != null) {
					res.close();
				}
				if (stmt != null) {
					stmt.close();
				}
				if (conn != null) {
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("select() error :" + collectorID + " " + facilityID + " EJBException ");
				throw new EJBException(e1.getMessage());
			}
		}
		m_log.debug("select() end   :" + collectorID + " " + facilityID);

		return ret;
	}
	
	/**
	 * 収集した性能値を収集IDで削除します。
	 * 
	 * 収集設定に紐付けられた性能値を一括で削除します。
	 * 
	 * @param collectorID　収集ID
	 * @throws SQLException
	 */
	public void delete(String collectorID)
	throws EJBException{
		
		m_log.debug("delete() start :" + collectorID );

		Connection conn = null;
		PreparedStatement stmt = null;
		
		try{
			conn = ConnectionManager.getConnectionManager().getConnection();
			//conn.setAutoCommit(false);
			//SQL文の定義
			String sql = "DELETE FROM CC_CALCULATED_DATA WHERE COLLECTORID = ? ";
			//SQL文のセット
			stmt = conn.prepareStatement(sql);
			
			stmt.setString(1,collectorID);
			
			stmt.executeUpdate();
		} catch (SQLException e) {
			m_log.error("delete() error :" + collectorID + " EJBException ");
			throw new EJBException(e.getMessage());
		}  finally{
			
			//コネクション、結果セット、プリペアドステートメントのクロース
			try {
				if(stmt != null){
					stmt.close();
				}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("delete() error :" + collectorID + " EJBException ");
				throw new EJBException(e1.getMessage());
			}
			m_log.debug("delete() end   :" + collectorID );
		}	
	}
	
	/**
	 * 収集した性能値をインサートします。
	 * 
	 * @param collectorID　収集ID
	 * @param data 計算済み性能値情報
	 * @throws SQLException
	 */
	public void insert(String collectorID, CollectedDataInfo data)
	throws EJBException{
		
		m_log.debug("insert() start : " + collectorID +
				" " + data.getItemCode() +
				" " + data.getDisplayName() +
				" " + data.getFacilityId() +
				" " + data.getDate()
				);
		
		Connection conn = null;
		PreparedStatement stmt = null;
		Timestamp time = new Timestamp(data.getDate().getTime());
		
		try{
			conn = ConnectionManager.getConnectionManager().getConnection();
			//SQL文の定義
			String sql = "INSERT INTO CC_CALCULATED_DATA " +
			"(COLLECTORID, " +
			"ITEM_CODE, " +
			"DISPLAY_NAME, " +
			"DATE_TIME, " +
			"FACILITYID, " +
			"VALUE) " +
			"values (?,?,?,?,?,?);";
			
			//SQL文のセット
			stmt = conn.prepareStatement(sql);
			
			stmt.setString(1, collectorID);
			stmt.setString(2, data.getItemCode());
			stmt.setString(3, data.getDisplayName());
			stmt.setTimestamp(4, time);
			stmt.setString(5, data.getFacilityId());
			if(Double.isNaN(data.getValue())){
				stmt.setNull(6, Types.DOUBLE);  // 性能値が算出不能だった場合はnullを書き込む
			} else {
				stmt.setDouble(6, data.getValue());
			}
			
			int row = stmt.executeUpdate();
			
			if (row != 1) {
				String msg = "result row is not 1";
				throw new SQLException(msg);
			}
		} catch (SQLException e) {
			m_log.error("insert() error :" + collectorID + " EJBException ", e);
			throw new EJBException(e.getMessage());
		}  finally{
			
			//コネクション、結果セット、プリペアドステートメントのクロース
			try {
				if(stmt != null){
					stmt.close();}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("insert() error :" + collectorID + " EJBException ", e1);
				throw new EJBException(e1.getMessage());
			}
			m_log.debug("insert() end   : " + collectorID +
					" " + data.getItemCode() +
					" " + data.getDisplayName() +
					" " + data.getFacilityId());
		}	
	}
	/**
	 * 保存期間を過ぎた実績値を削除します。
	 * 
	 * @param collectorID 収集ID
	 * @param startDate 開始時刻
	 * @throws EJBException
	 */
	public void deleteOverPresave(String collectorID , Date startDate)
		throws EJBException{
			
			m_log.debug("deleteOrverPresave() start :" + collectorID + " " + startDate.toString());

			Connection conn = null;
			PreparedStatement stmt = null;
			Timestamp time = new Timestamp(startDate.getTime());
			
			try{
				conn = ConnectionManager.getConnectionManager().getConnection();
				//conn.setAutoCommit(false);
				//SQL文の定義
				String sql = "DELETE FROM CC_CALCULATED_DATA WHERE COLLECTORID = ? AND DATE_TIME < ?";
				//SQL文のセット
				stmt = conn.prepareStatement(sql);
				
				stmt.setString(1,collectorID );
				stmt.setTimestamp(2,time);
				
				stmt.executeUpdate();
			} catch (SQLException e) {
				m_log.debug("deleteOrverPresave() error :" + collectorID + " " + startDate.toString() + "  EJBException ");

				throw new EJBException(e.getMessage());
			}  finally{
				
				//コネクション、結果セット、プリペアドステートメントのクロース
				try {
					if(stmt != null){
						stmt.close();
					}
					if(conn != null){
						conn.close();
					}
				} catch (SQLException e1) {
					m_log.debug("deleteOrverPresave() error :" + collectorID + " " + startDate.toString() + "  EJBException ");
					throw new EJBException(e1.getMessage());
				}
				m_log.debug("deleteOrverPresave() end :" + collectorID + " " + startDate.toString() + "  EJBException ");
			}	
	}

	/**
	 * 収集回数を再カウントします。
	 * 
	 * @param collectorID
	 * @return 再カウントした結果の収集件数（失敗した場合は負数が返る）
	 * @throws EJBException
	 */
		public int recountFromDB(String collectorID)throws EJBException{
			
			m_log.debug("recountFromDB() start :" + collectorID );
			
			Connection conn = null;
			PreparedStatement stmt =null;
			ResultSet res = null; 
			int ret = -1;
			try {
				conn = ConnectionManager.getConnectionManager().getConnection();
				
				// 指定された収集IDの任意の1件を取り出し、そのitem_code、facilityid、display_nameを取得する。
				final String preQuery = "SELECT item_code, facilityid, display_name FROM cc_calculated_data" + 
										" where collectorid = ? LIMIT 1";
				stmt = conn.prepareStatement(preQuery);
				stmt.setString(1, collectorID);
				res = stmt.executeQuery();
				if (!res.next()) {
					// 1件もとれない（処理に失敗した）場合は負数を返す
					return -1;
				}
				final String item_code = res.getString("item_code");
				final String facilityid = res.getString("facilityid");
				final String display_name = res.getString("display_name");
				
				// 指定された収集IDで何回収集が行われたかを取得する。
				// 収集の開始から現時点に至るまでに、item_code/facilityidが変わることは
				// ありえないことと、同一時間帯にitem_code（収集対象の情報種類）とfacilityid
				// （収集対象ノード）の数だけ重複して情報を収集することから、ある特定のitem_code
				// とfacilityidに絞ってその件数を数えれば、収集を実施した回数がわかることになる。
				// デバイス別の収集項目があることを踏まえ、条件にdisplay_nameを加える。
				final String cntQuery = "SELECT COUNT(*) FROM cc_calculated_data " +
					"WHERE collectorid = ? AND item_code = ? AND facilityid = ? AND display_name = ?";
				stmt = conn.prepareStatement(cntQuery);
				stmt.setString(1, collectorID);
				stmt.setString(2, item_code);
				stmt.setString(3, facilityid);
				stmt.setString(4, display_name);
				res = stmt.executeQuery();
				if (!res.next()) {
					// 1件もとれない（処理に失敗した）場合は負数を返す
					return -1;
				}
				ret = res.getInt(1);
				
			} catch (SQLException e) {
				m_log.error("recountFromDB() error :" + collectorID + " EJBException ",e);
				throw new EJBException(e.getMessage());
			}  finally{
				
				//コネクション、結果セット、プリペアドステートメントのクロース
				try {
					if(res != null){
						res.close();
					}
					if(stmt != null){
						stmt.close();
					}
					if(conn != null){
						conn.close();
					}
				} catch (SQLException e1) {
					m_log.error("recountFromDB() error :" + collectorID + " EJBException ");
					throw new EJBException(e1.getMessage());
				}
				m_log.debug("recountFromDB() end   :" + collectorID + " ");
			}
			
			return ret;
		}
	}
