/*
 
 Copyright (C) 2006 NTT DATA Corporation
 
 This program is free software; you can redistribute it and/or
 Modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation, version 2.
 
 This program is distributed in the hope that it will be
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 */

package com.clustercontrol.logagent;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
import java.util.Arrays;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.clustercontrol.bean.PriorityConstant;
import com.clustercontrol.bean.YesNoConstant;
import com.clustercontrol.logagent.util.AgentProperties;
import com.clustercontrol.logagent.util.LoggerSyslog;
import com.clustercontrol.util.Messages;

/**
 * ログ転送スレッドクラス<BR>
 * 
 * @version 3.0.0
 * @since 2.1.0
 */
public class TransferLogThread extends Thread {

	private final static String UNCHANGED_STATS_PERIOD = "unchanged.stats.period";
	
	private final static String FIRST_PART_DATA_CHECK_PERIOD = "first.part.data.check.period";

	private final static String FIRST_PART_DATA_CHECK_SIZE = "first.part.data.check.size";
	
	private final static String FILE_MAX_SIZE = "file.max.size";
	
	private final static String LOG_ENCODING = "log.file.encoding";

	private final static String PROGRAM = "log.msg.program";
	
	public static final String MESSAGE_ID_INFO = "001";

	public static final String MESSAGE_ID_WARNING = "002";

	public static final String MESSAGE_ID_CRITICAL = "003";

	public static final String MESSAGE_ID_UNKNOWN = "004";

	public static final String HINEMOS_LOG_AGENT = "hinemos_log_agent";

	protected TransferLogManager m_transferLogManager;

	/** ログファイル名 */
	protected String m_filePath;

	/** 動作間隔（ミリ秒） */
	protected int m_runInterval;

	/** 最初にファイルをチェック */
	protected int m_existenceFlg = YesNoConstant.TYPE_NO;

	/** ファイル変更チェック期間設定（ミリ秒） */
	protected int m_unchangedStatsPeriod = 0;
	
	/** ファイル変更詳細チェック（冒頭データ比較）期間（ミリ秒） */
	protected int m_firstPartDataCheckPeriod = 0;

	/** ファイル変更詳細チェック（冒頭データ比較）サイズ（byte） */
	protected int m_firstPartDataCheckSize = 0;
	
	/** 上限ファイルサイズ設定（byte） */
	protected long m_fileMaxSize = 0L;

	/** ログ先頭に定義するプログラム名 */
	protected String m_program = HINEMOS_LOG_AGENT;
	
	/** ログ転送停止フラグ */
	protected boolean m_stopFlg = false;

	// Syslog転送用ロガー
	protected LoggerSyslog m_syslog = null;

	// ロガー
	static Log log = LogFactory.getLog(TransferLogThread.class);
	
	// ログファイルのエンコーディング
	private String m_logEncoding = "UTF-8";
	
	/**
	 * コンストラクタ
	 * 
	 * @param queue
	 * @param props
	 * @param path
	 *            転送対象ログファイル
	 * @param interval
	 *            動作間隔（秒）
	 * @param flg
	 *            最初にファイルをチェック
	 */
	public TransferLogThread(TransferLogManager transferLogManager,
			String path, int interval, int flg) {

		m_transferLogManager = transferLogManager;
		m_filePath = path;
		m_runInterval = interval * 1000; // ミリ秒に変換
		m_existenceFlg = flg;

		// ファイル変更チェック期間（秒）
		String sleepInterval = AgentProperties.getProperty(UNCHANGED_STATS_PERIOD, "5");
		log.info(UNCHANGED_STATS_PERIOD + " = " + sleepInterval + " sec");
		try {
			this.m_unchangedStatsPeriod = Integer.parseInt(sleepInterval) * 1000;
		} catch (NumberFormatException e) {
			log.error("TransferLogManager() : " + UNCHANGED_STATS_PERIOD, e);
		}
		log.debug(UNCHANGED_STATS_PERIOD + " = " + m_unchangedStatsPeriod);


		// ファイル変更詳細チェック（冒頭データ比較）期間（秒）
		String firstPartDataCheckPeriod = AgentProperties.getProperty(FIRST_PART_DATA_CHECK_PERIOD, "300");
		try {
			this.m_firstPartDataCheckPeriod = Integer.parseInt(firstPartDataCheckPeriod) * 1000;
		} catch (NumberFormatException e) {
			log.error("TransferLogManager() : " + FIRST_PART_DATA_CHECK_PERIOD, e);
		}
		log.debug(FIRST_PART_DATA_CHECK_PERIOD + " = " + m_firstPartDataCheckPeriod);


		// ファイル変更詳細チェック（冒頭データ比較）サイズ（byte）
		String firstPartDataCheckSize = AgentProperties.getProperty(FIRST_PART_DATA_CHECK_SIZE, "256");
		try {
			this.m_firstPartDataCheckSize = Integer.parseInt(firstPartDataCheckSize);
		} catch (NumberFormatException e) {
			log.error("TransferLogManager() : " + FIRST_PART_DATA_CHECK_SIZE, e);
		}
		log.debug(FIRST_PART_DATA_CHECK_SIZE + " = " + m_firstPartDataCheckSize);

		// 上限ファイルサイズ（byte）
		String fileMaxSize = AgentProperties.getProperty(FILE_MAX_SIZE, "2147483648");
		log.info(FILE_MAX_SIZE + " = " + fileMaxSize + " byte");
		try {
			this.m_fileMaxSize = Long.parseLong(fileMaxSize);
		} catch (NumberFormatException e) {
			log.error("TransferLogManager() : " + FILE_MAX_SIZE, e);
		}
		log.debug(FILE_MAX_SIZE + " = " + m_fileMaxSize);

		// プログラム名を設定
		m_program= AgentProperties.getProperty(PROGRAM, HINEMOS_LOG_AGENT);
		if ("".equals(m_program)) {
			m_program = HINEMOS_LOG_AGENT;
		}
		log.debug(PROGRAM + " = " + m_program);

		// ログファイルのエンコーディングを設定
		m_logEncoding =  AgentProperties.getProperty(LOG_ENCODING);
		
		// プロパティファイルに設定がない場合
		if(m_logEncoding == null){
			m_logEncoding = System.getProperty("file.encoding"); 
		}
		log.info(LOG_ENCODING + " = " + m_logEncoding);
		
		m_syslog = new LoggerSyslog();
	}

	/**
	 * スレッドの動作メソッド<BR>
	 * 
	 */
	/*
	 * (非 Javadoc)
	 * 
	 * @see java.lang.Thread#run()
	 */
	@Override
	public void run() {
		log.info("monitor start.  logfile : " + m_filePath 
				+ "  interval : " + m_runInterval + "ms"
				+ "  logfile encoding : " + m_logEncoding
				+ "  syslog encoding : " + System.getProperty("file.encoding"));

		long filesize = 0;
		long tmp_filesize = 0;
		long n_unchanged_stats = 0; // ファイルチェック時に、ファイルに変更がなかった回数
		// String carryOver="";
		byte[] cbuf = new byte[1024]; // ファイルからの読み出し分を保持するバッファ
		byte[] carryOverBuf = new byte[0]; // 文字化け対策用に繰り越すバッファ
		byte[] appendedBuf = null; // 繰り越し分にファイルからの読み出し分を追加したバッファ

		File name = new File(this.m_filePath); // 監視対象ログファイル

		long lastDataCheck = System.currentTimeMillis(); // 最終詳細チェック（冒頭データ比較）実行時刻
		
		// ファイルオープン
		RandomAccessFile fr = openFile(name, true, false);
		if (fr == null) {
			return;
		}

		String logPrefix = m_program + "(" + this.m_filePath + "):";

		while (true) {

			// スリープ
			try {
				Thread.sleep(this.m_runInterval);
			} catch (InterruptedException e) {
			}

			if (this.m_stopFlg) {
				closeFile(fr);
				return;
			}

			try {
				tmp_filesize = fr.length(); // 現在監視しているファイのサイズを取得・・・（１）

				if (filesize == tmp_filesize) {

					// ファイルサイズがm_unchangedStatsPeriod秒間以上変わらなかったら、ファイル切り替わりチェック
					if ((++n_unchanged_stats * this.m_runInterval) >= this.m_unchangedStatsPeriod) {
						log.debug("run() : " + m_filePath + " check log rotation");
						boolean logrotate = false;
						
						// ログローテートされているかチェックする
						// 従来の判定ロジック：
						// 　現在監視しているファイルのサイズと本来監視すべきファイルのサイズが異なっている場合、
						// 　mv方式によりローテートされたと判断する。
						if (fr.length() != name.length()) { // ・・・（２）
							log.debug("run() : " + m_filePath + " file size not match");
							logrotate = true;
						} else if (tmp_filesize > 0 && m_firstPartDataCheckPeriod > 0 &&
								(System.currentTimeMillis() - lastDataCheck) > m_firstPartDataCheckPeriod){
							log.debug("run() : " + m_filePath + " check first part of file");

							// 追加された判定ロジック：
							// 　現在監視しているファイルのサイズと本来監視すべきファイルのサイズが同じであっても、
							// 　mv方式によりローテートされた可能性がある。
							// 　ファイルの冒頭部分を確認することで、ローテートされたことを確認する。
							byte[] refFirstPartOfFile = new byte[m_firstPartDataCheckSize];
							Arrays.fill(refFirstPartOfFile, (byte)0);  // 全ての要素を0で初期化
							byte[] newFirstPartOfFile = new byte[m_firstPartDataCheckSize];
							Arrays.fill(newFirstPartOfFile, (byte)0);  // 全ての要素を0で初期化

							// ファイル名指定で新たにオープンするファイル
							RandomAccessFile newFile = null;
							
							// 現在監視しているファイルの冒頭データを取得
							long bufSeek = 0;
							try {
								bufSeek = fr.getFilePointer();

								// ファイル冒頭に移動する
								fr.seek(0);
								if (fr.read(refFirstPartOfFile) > 0) {
									if(log.isDebugEnabled()){
										try {
											log.debug("run() : " + m_filePath + " refFirstPartOfFile : "
													+ new String(refFirstPartOfFile, 
															0, 
															tmp_filesize < refFirstPartOfFile.length ? (int)tmp_filesize : refFirstPartOfFile.length, 
																	m_logEncoding));
										} catch (Exception e) {
											log.error("run() : " + m_filePath + " " + e.getMessage(), e);
										}
									}
									
									// 再度ファイル名指定でファイルを開く
									newFile = new RandomAccessFile(name, "r");
									if (newFile.read(newFirstPartOfFile) > 0) {
										if(log.isDebugEnabled()){
											try {
												log.debug("run() : " + m_filePath + " newFirstPartOfFile : "
														+ new String(newFirstPartOfFile, 
																0, 
																tmp_filesize < newFirstPartOfFile.length ? (int)tmp_filesize : newFirstPartOfFile.length, 
																		m_logEncoding));
											} catch (Exception e) {
												log.error("run() : " + m_filePath + " " + e.getMessage(), e);
											}
										}
										
										// ファイルの冒頭部分が異なれば別ファイルと判定
										if (!Arrays.equals(refFirstPartOfFile, newFirstPartOfFile)) {
											log.debug("run() : " + m_filePath + " log rotation detected");
											logrotate = true;
										}
									}
								}
							} catch (Exception e) {
								log.error("run() : " + m_filePath + " " + e.getMessage(), e);
							} finally {
								// ファイルのオフセットを元に戻す
								if (bufSeek != 0) {
									fr.seek(bufSeek);
								}
								
								// オープンしたファイルをクローズ
								if(newFile != null){
									try {
										newFile.close();
									} catch (Exception e) {
										log.error("run() : " + m_filePath + " " + e.getMessage(), e);
									}
								}
							}
							// 最終詳細チェック（ファイルデータ比較）実行時刻を設定
							lastDataCheck = System.currentTimeMillis();
						}

						// ログローテートされたと判定された場合
						if(logrotate){
							// 再度ファイルサイズの増加を確認する
							// ローテートされたと判定されたが、実は、ローテートされておらず、
							//  （１）の時点では最終ログ読み込み時からログが出力されていないためサイズ（filesize）は同一であったが、
							//  （２）の判定直前にログが出力された場合は、ローテートされたと誤検知するため、
							// 再度サイズ比較を行う。
							if (tmp_filesize == fr.length()) {
								log.info(this.m_filePath + " : file changed");
								closeFile(fr);

								// ファイルオープン
								fr = openFile(name, false, false);
								if (fr == null) {
									return;
								}

								filesize = 0;
								// carryOver = "";
								carryOverBuf = new byte[0];
							}

						}
						n_unchanged_stats = 0;
					}
					continue;
				}

				n_unchanged_stats = 0;

				if (filesize < tmp_filesize) {
					// デバッグログ
					if (log.isDebugEnabled()) {
						log
								.debug("run() : " + m_filePath + " filesize "
										+ filesize + "    tmp_filesize "
										+ tmp_filesize);
					}

					// 加分読み込み
					int read;
					// StringBuffer sb = new StringBuffer(carryOver);
					StringBuffer sb = new StringBuffer();
					while ((read = fr.read(cbuf)) != -1) {

						// //
						// UTF-8を含むログで文字化けが発生するため修正
						//
						// 最後の改行コードを検索し、最後の改行コード以降は次回に繰越て処理する。
						// carryOverStartには改行コードの次のコードのインデックスが格納される。
						int carryOverStart = read;
						boolean returnCode = false;
						for (int i = read - 1; i >= 0; i--) {
							if (cbuf[i] == '\n' || cbuf[i] == '\r') {
								carryOverStart = i + 1;
								returnCode = true;
								break;
							}
						}

						// デバッグログ
						if (log.isDebugEnabled()) {
							log.debug("run() : " + m_filePath + " read " + read
									+ "    carryOverStart " + carryOverStart);
						}

						// 今回出力処理する分のバッファを作成
						// 前回の繰越分と今回ファイルから読み出したデータのうち
						// 最後の改行までのデータをマージする。
						appendedBuf = new byte[carryOverBuf.length
								+ carryOverStart];
						// for(int i = 0; i<carryOverBuf.length; i++){
						// appendedBuf[i] = carryOverBuf[i];
						// }
						ByteBuffer.wrap(carryOverBuf).get(appendedBuf, 0,
								carryOverBuf.length);

						// int j=0;
						// for(int i = carryOverBuf.length;
						// i<appendedBuf.length; i++){
						// appendedBuf[i] = cbuf[j++];
						// }
						ByteBuffer.wrap(cbuf).get(appendedBuf,
								carryOverBuf.length, carryOverStart);

						// デバッグログ
						if (log.isDebugEnabled()) {
							log
									.debug("run() : " + m_filePath
											+ " appendedBuf size "
											+ appendedBuf.length);
						}

						// 改行コードが含まれない場合
						if (!returnCode) {
							// 今回ファイルから読み込んだものを含めて全て次回へ繰り越す。
							// 出力処理は実施しない。
							log.debug("run() : " + m_filePath
									+ " return code is not exist");
							carryOverBuf = new byte[appendedBuf.length];
							ByteBuffer.wrap(appendedBuf).get(carryOverBuf);
							continue;
						}

						// 繰越データ用バッファを作成
						carryOverBuf = new byte[(read - carryOverStart)];
						try {
							// 最後が改行コード以降にデータがある場合は、次回の処理にまわす
							if (read > carryOverStart) {
								// デバッグログ
								if (log.isDebugEnabled()) {
									log.debug("run() : " + m_filePath
											+ " carryOverBuf size "
											+ carryOverBuf.length);
								}
								ByteBuffer.wrap(cbuf, carryOverStart,
										carryOverBuf.length).get(carryOverBuf);
							}
						} catch (Exception e) {
							// デバッグログ
							log.error("run() : " + e.getMessage(), e);
						}
						// //

						try {
//							sb.append(new String(appendedBuf, 0,
//									appendedBuf.length, System
//											.getProperty("file.encoding")));
							sb.append(new String(appendedBuf, 0,
									appendedBuf.length, m_logEncoding));							
						} catch (UnsupportedEncodingException e) {
							log.error("run() : " + m_filePath + " "
									+ e.getMessage());
						}
					}

					String tmpString = sb.toString();
					String[] result = tmpString.split("\\n");
					// デバッグログ
					if (log.isDebugEnabled()) {
						log.debug("run() : " + m_filePath + " " + sb);
						log.debug("run() : " + m_filePath + " size "
								+ sb.length());
						log.debug("run() : " + m_filePath + " result size "
								+ result.length);
					}

					// 読み込み文字列のサイズが0でない場合は処理する
					if (sb.length() != 0) {
						for (int x = 0; x < result.length; x++) {
							m_syslog.log(logPrefix + result[x]);
						}
					}
					// 改行で終わらない場合の繰越
					// if( tmpString.endsWith("\n") ){
					// m_syslog.log(logPrefix + result[result.length-1]);
					// carryOver = "";
					// }else{
					// carryOver = result[result.length-1];
					// }

					filesize = tmp_filesize;
					if(log.isDebugEnabled()){
						log.debug("run() : " + m_filePath + " filesize = " + filesize + ", FilePointer = " + fr.getFilePointer());
					}

				} else if (filesize > tmp_filesize) {
					// 切詰
					log.info(this.m_filePath + " : file size becomes small");
					fr.seek(0);
					filesize = 0;
					// carryOverBu = "";
					carryOverBuf = new byte[0];
				}

			} catch (IOException e) {
				log.error("run() : " + e.getMessage());
				Object[] args = { this.m_filePath };
				sendMessage(PriorityConstant.TYPE_WARNING, 
						Messages.getString("log.agent"), 
						MESSAGE_ID_WARNING,
						Messages.getString("message.log.agent.4"), 
						Messages.getString("message.log.agent.1", args) + "\n" + e.getMessage());

				// エラーが発生したのでファイルクローズし再オープン

				closeFile(fr);

				// ファイルオープン
				fr = openFile(name, false, true);
				if (fr == null) {
					return;
				}

				filesize = 0;
				// carryOver = "";
				carryOverBuf = new byte[0];
			}
		}
	}

	/**
	 * ログ転送を停止します。<BR>
	 * 
	 */
	public void requestStop() {
		this.m_stopFlg = true;
		this.interrupt();
		
		log.info("monitor stop.   logfile : " + m_filePath 
				+ "  interval : " + m_runInterval + "s"
				+ "  logfile encoding : " + m_logEncoding
				+ "  syslog encoding : " + System.getProperty("file.encoding"));
	}

	/**
	 * 監視管理情報へ通知
	 * 
	 * @param priority
	 *            重要度
	 * @param app
	 *            アプリケーション
	 * @param msgId
	 *            メッセージID
	 * @param msg
	 *            メッセージ
	 * @param msgOrg
	 *            オリジナルメッセージ
	 */
	private void sendMessage(int priority, String app, String msgId,
			String msg, String msgOrg) {

		m_transferLogManager.sendMessage(m_filePath, priority, app, msgId, msg,
				msgOrg);

	}

	/**
	 * 転送対象ログファイルクローズ
	 * 
	 */
	private void closeFile(RandomAccessFile fr) {

		if (fr != null) {
			try {
				fr.close();
			} catch (IOException e) {
				log.debug("run() : " + e.getMessage());
			}
		}
	}

	/**
	 * 転送対象ログファイルオープン
	 * 
	 */
	private RandomAccessFile openFile(File name, boolean init, boolean isOnErr) {

		boolean err = isOnErr;
		RandomAccessFile fr = null;
		// ファイルオープン
		while (true) {

			try {

				fr = new RandomAccessFile(name, "r");

				long filesize = fr.length();
				if (filesize > this.m_fileMaxSize) {
					// ファイルサイズが大きい場合、監視管理へ通知
					Object[] args1 = { this.m_filePath };
					Object[] args2 = { filesize };
					sendMessage(PriorityConstant.TYPE_INFO, 
							Messages.getString("log.agent"), 
							MESSAGE_ID_INFO,
							Messages.getString("message.log.agent.3"), 
							Messages.getString("message.log.agent.1", args1) + ", "	
								+ Messages.getString("message.log.agent.5",args2));
				}

				// ファイルポインタの設定
				if (init) {
					fr.seek(filesize);
				}

				return fr;

			} catch (FileNotFoundException e) {
				if (init && !err) {
					if (this.m_existenceFlg == YesNoConstant.TYPE_YES) {
						// 最初にファイルをチェックする場合、監視管理へ通知
						Object[] args = { this.m_filePath };
						sendMessage(PriorityConstant.TYPE_INFO,
								Messages.getString("log.agent"),
								MESSAGE_ID_INFO, 
								Messages.getString("message.log.agent.2"),
								Messages.getString("message.log.agent.1", args));
					}
					err = true;
				}

			} catch (SecurityException e) {
				if (!err) {
					// 監視管理へ通知
					Object[] args = { this.m_filePath };
					sendMessage(PriorityConstant.TYPE_WARNING, 
							Messages.getString("log.agent"),
							MESSAGE_ID_WARNING, 
							Messages.getString("message.log.agent.4"),
							Messages.getString("message.log.agent.1", args) + "\n" + e.getMessage());
					err = true;
				}
			} catch (IOException e) {
				if (!err) {
					// 監視管理へ通知
					Object[] args = { this.m_filePath };
					sendMessage(PriorityConstant.TYPE_INFO, 
							Messages.getString("log.agent"), 
							MESSAGE_ID_INFO,
							Messages.getString("message.log.agent.4"), 
							Messages.getString("message.log.agent.1", args));
					err = true;
				}

				closeFile(fr);
			}

			// 初期処理は初回のみとする
			// 初回オープンに失敗後、ファイルをオープンすると、ファイルのリードポインタが
			// ファイルのオープン時に設定されてしまうため
			init = false;

			// スリープ
			try {
				Thread.sleep(this.m_runInterval);
			} catch (InterruptedException ie) {
			}

			if (this.m_stopFlg) {
				return null;
			}

		}

	}

	public void setCondition(int chkExistence, int interval) {
		m_existenceFlg = chkExistence;
		m_runInterval = interval * 1000;
	}

	public static void main(String[] args) {

	}

}